@@ -1,14 +1,6 @@
+#include <string.h>
 #include "curve.h"
-
-//I define "common" curves (abbr. "cc") to be those
-//of the form y^2 = x^3 + ax + b defined over a field with
-//characteristic > 3
-
-struct common_curve_s {
-    element_t a, b;
-};
-typedef struct common_curve_s common_curve_t[1];
-typedef struct common_curve_s *common_curve_ptr;
+#include "parse.h"
 
 static void cc_random(point_ptr p)
 {
@@ -133,8 +125,17 @@ static void cc_mul(point_ptr r, mpz_ptr n, point_ptr p)
     point_clear(result);
 }
 
+static void cc_clear(curve_ptr c)
+{
+    common_curve_ptr cc = c->data;
+    element_clear(cc->a);
+    element_clear(cc->b);
+    free(c->data);
+}
+
 void curve_init_cc_j(curve_ptr c, element_ptr j)
 //assumes j != 0, 1728
+//TODO: call curve_init_cc_ab instead of duplicating code
 {
     common_curve_ptr cc;
 
@@ -143,6 +144,7 @@ void curve_init_cc_j(curve_ptr c, element_ptr j)
     c->add = cc_add;
     c->mul = cc_mul;
     c->data = malloc(sizeof(common_curve_t));
+    c->curve_clear = cc_clear;
     cc = c->data;
     element_init(cc->a, c->field);
     element_init(cc->b, c->field);
@@ -155,15 +157,8 @@ void curve_init_cc_j(curve_ptr c, element_ptr j)
     element_add(cc->b, cc->a, cc->a);
     //a = 3 j / (1728 - j)
     element_add(cc->a, cc->a, cc->b);
-    printf("a = ");
-    element_out_str(stdout, cc->a);
-    printf(", b = ");
-    element_out_str(stdout, cc->b);
-    printf("\n");
 }
 
-//TODO: curve_clear_cc_j
-
 void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
 {
     common_curve_ptr cc;
@@ -172,6 +167,7 @@ void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
     c->add = cc_add;
     c->mul = cc_mul;
     c->data = malloc(sizeof(common_curve_t));
+    c->curve_clear = cc_clear;
     cc = c->data;
     element_init(cc->a, c->field);
     element_init(cc->b, c->field);
@@ -180,6 +176,8 @@ void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
 }
 
 void twist_curve(curve_ptr c)
+    //WARNING: existing points will no longer lie on c
+    //as this modifies c in place
 {
     common_curve_ptr cc = c->data;
     element_ptr nqr = c->field->nqr;
@@ -400,8 +398,8 @@ void slow_miller(element_t res, mpz_t q, point_t P, point_t Q)
 
     point_set(Z, P);
 
-    element_set_si(v, 1);
-    element_set_si(vd, 1);
+    element_set1(v);
+    element_set1(vd);
     m = mpz_sizeinbase(q, 2) - 2;
 
     while(m >= 0) {
@@ -562,8 +560,8 @@ void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q)
     point_extend(P, Pbase, (Q->curve->field));
     point_set(Z, P);
 
-    element_set_si(v, 1);
-    element_set_si(vd, 1);
+    element_set1(v);
+    element_set1(vd);
     m = mpz_sizeinbase(q, 2) - 2;
 
     while(m >= 0) {
@@ -588,3 +586,115 @@ void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q)
     point_clear(Z);
     point_clear(P);
 }
+
+void cc_param_init(cc_param_ptr param)
+{
+    mpz_init(param->q);
+    mpz_init(param->n);
+    mpz_init(param->h);
+    mpz_init(param->r);
+    mpz_init(param->a);
+    mpz_init(param->b);
+    mpz_init(param->nk);
+    mpz_init(param->hk);
+}
+
+void cc_param_clear(cc_param_ptr param)
+{
+    mpz_clear(param->q);
+    mpz_clear(param->n);
+    mpz_clear(param->h);
+    mpz_clear(param->r);
+    mpz_clear(param->a);
+    mpz_clear(param->b);
+    mpz_clear(param->nk);
+    mpz_clear(param->hk);
+}
+
+void cc_param_set(cc_param_ptr x, cc_param_ptr y)
+{
+    mpz_set(x->q, y->q);
+    mpz_set(x->n, y->n);
+    mpz_set(x->h, y->h);
+    mpz_set(x->r, y->r);
+    mpz_set(x->a, y->a);
+    mpz_set(x->b, y->b);
+    mpz_set(x->nk, y->nk);
+    mpz_set(x->hk, y->hk);
+}
+
+void cc_param_out_str(FILE *stream, cc_param_ptr p)
+{
+    void out(char *s, mpz_t z)
+    {
+	fprintf(stream, "%s ", s);
+	mpz_out_str(stream, 0, z);
+	fprintf(stream, "\n");
+    }
+
+    out("q", p->q);
+    out("n", p->n);
+    out("h", p->h);
+    out("r", p->r);
+    out("a", p->a);
+    out("b", p->b);
+    printf("k %d\n", p->k);
+    out("nk", p->nk);
+    out("hk", p->hk);
+}
+
+void cc_param_inp_str(cc_param_ptr p, FILE *stream)
+{
+    token_t tok;
+    mpz_t k;
+    mpz_ptr z;
+
+    token_init(tok);
+    mpz_init(k);
+    for (;;) {
+	token_get(tok, stream);
+	if (tok->type != token_word) {
+	    break;
+	}
+	z = NULL;
+	if (strlen(tok->s) == 1) {
+	    switch(tok->s[0]) {
+		case 'q':
+		    z = p->q;
+		    break;
+		case 'n':
+		    z = p->n;
+		    break;
+		case 'r':
+		    z = p->r;
+		    break;
+		case 'h':
+		    z = p->h;
+		    break;
+		case 'k':
+		    z = k;
+		    break;
+		case 'a':
+		    z = p->a;
+		    break;
+		case 'b':
+		    z = p->b;
+		    break;
+	    }
+	} else if (!strcmp(tok->s, "nk")) {
+	    z = p->nk;
+	} else if (!strcmp(tok->s, "hk")) {
+	    z = p->hk;
+	}
+	token_get(tok, stream);
+	if (tok->type != token_word) {
+	    break;
+	}
+	if (z) {
+	    mpz_set_str(z, tok->s, 0);
+	}
+    }
+    p->k = mpz_get_ui(k);
+    mpz_clear(k);
+    token_clear(tok);
+}
