@@ -29,6 +29,7 @@ struct field_s {
     void (*set_mpz)(element_ptr, mpz_ptr);
     void (*set)(element_ptr, element_ptr);
     void (*set0)(element_ptr);
+    void (*set1)(element_ptr);
     void (*out_str)(FILE *stream, element_ptr);
     void (*add)(element_ptr, element_ptr, element_ptr);
     void (*sub)(element_ptr, element_ptr, element_ptr);
@@ -92,6 +93,11 @@ static inline void element_set0(element_ptr e)
     e->field->set0(e);
 }
 
+static inline void element_set1(element_ptr e)
+{
+    e->field->set1(e);
+}
+
 static inline void element_set(element_ptr x, element_ptr a)
 {
     x->field->set(x, a);
