@@ -93,9 +93,9 @@ void poly_set_coeff(element_ptr e, element_ptr a, int n)
 
 void poly_setx(element_ptr f)
 {
-    element_set0(f);
     poly_alloc(f, 2);
-    element_set_si(poly_coeff(f, 1), 1);
+    element_set0(poly_coeff(f, 0));
+    element_set1(poly_coeff(f, 1));
 }
 
 static void poly_set(element_ptr dst, element_ptr src)
@@ -238,7 +238,7 @@ void poly_random_monic(element_ptr f, int deg)
     for (i=0; i<deg; i++) {
 	element_random(poly_coeff(f, i));
     }
-    element_set_si(poly_coeff(f, i), 1);
+    element_set1(poly_coeff(f, i));
 }
 
 static void polymod_random(element_ptr f)
@@ -265,7 +265,7 @@ static void polymod_mul(element_ptr res, element_ptr f, element_ptr g)
     n = poly_degree(p->poly);
 
     poly_mul(prod, f, g);
-    
+
     n1 = poly_degree(prod);
     for (i=n; i<=n1; i++) {
 	poly_const_mul(p0, poly_coeff(prod, i), p->xpwr->item[i-n]);
@@ -283,6 +283,16 @@ static void poly_set0(element_ptr e)
     poly_alloc(e, 0);
 }
 
+static void poly_set1(element_ptr e)
+{
+    poly_element_ptr p = e->data;
+    element_ptr e0;
+
+    poly_alloc(e, 1);
+    e0 = p->coeff->item[0];
+    element_set1(e0);
+}
+
 static int poly_is0(element_ptr e)
 {
     poly_element_ptr p = e->data;
@@ -382,7 +392,7 @@ void poly_invert(element_ptr res, element_ptr f, element_ptr m)
     element_init(r2, res->field);
     element_init(inv, poly_base_field(res));
     element_set0(b0);
-    element_set_si(b1, 1);
+    element_set1(b1);
     element_set(r0, m);
     element_set(r1, f);
 
@@ -452,17 +462,18 @@ void field_init_poly(field_ptr f, field_ptr base_field)
     f->field_clear = field_clear_poly;
     f->init = poly_init;
     f->clear = poly_clear;
-    f->set_si = poly_set_si; 
-    f->set_mpz = poly_set_mpz; 
-    f->out_str = poly_out_str; 
-    f->set = poly_set; 
-    f->add = poly_add; 
-    f->is0 = poly_is0; 
-    f->is1 = poly_is1; 
-    f->set0 = poly_set0; 
-    f->sub = poly_sub; 
-    f->neg = poly_neg; 
-    f->mul = poly_mul; 
+    f->set_si = poly_set_si;
+    f->set_mpz = poly_set_mpz;
+    f->out_str = poly_out_str;
+    f->set = poly_set;
+    f->add = poly_add;
+    f->is0 = poly_is0;
+    f->is1 = poly_is1;
+    f->set0 = poly_set0;
+    f->set1 = poly_set1;
+    f->sub = poly_sub;
+    f->neg = poly_neg;
+    f->mul = poly_mul;
     f->cmp = poly_cmp;
     mpz_init(f->order);
 }
@@ -515,7 +526,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a)
     element_init(e0, a->field);
 
     poly_alloc(f, 3);
-    element_set_si(poly_coeff(f, 2), 1);
+    element_set1(poly_coeff(f, 2));
     element_neg(poly_coeff(f, 0), a);
 
     mpz_sub_ui(z, a->field->order, 1);
@@ -526,7 +537,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a)
 	element_ptr e1, e2;
 
 	poly_alloc(r, 2);
-	element_set_si(poly_coeff(r, 1), 1);
+	element_set1(poly_coeff(r, 1));
 	x = poly_coeff(r, 0);
 	element_random(x);
 	element_mul(e0, x, x);
@@ -534,7 +545,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a)
 	    element_set(res, x);
 	    break;
 	}
-	element_set_si(s, 1);
+	element_set1(s);
 	//TODO: this can be optimized greatly
 	//since we know r has to form ax + b
 	for (i = mpz_sizeinbase(z, 2) - 1; i >=0; i--) {
@@ -560,7 +571,7 @@ static void polymod_sqrt(element_ptr res, element_ptr a)
 	    }
 	}
 	if (poly_degree(s) < 1) continue;
-	element_set_si(e0, 1);
+	element_set1(e0);
 	e1 = poly_coeff(s, 0);
 	e2 = poly_coeff(s, 1);
 	element_add(e1, e1, e0);
@@ -594,7 +605,7 @@ void poly_make_monic(element_t f, element_t g)
     for (i=0; i<n-1; i++) {
 	element_mul(poly_coeff(f, i), poly_coeff(g, i), e0);
     }
-    element_set_si(e0, 1);
+    element_set1(e0);
 }
 
 void compute_x_powers(darray_ptr xpwr, element_ptr poly)
@@ -653,13 +664,14 @@ void field_init_polymod_no_nqr(field_ptr f, element_ptr poly)
     f->init = poly_init;
     f->clear = poly_clear;
     f->set_si = poly_set_si;
-    f->set_mpz = poly_set_mpz; 
+    f->set_mpz = poly_set_mpz;
     f->out_str = poly_out_str;
     f->set = poly_set;
     f->add = poly_add;
     f->is0 = poly_is0;
-    f->is1 = poly_is1; 
+    f->is1 = poly_is1;
     f->set0 = poly_set0;
+    f->set1 = poly_set1;
     f->sub = poly_sub;
     f->neg = poly_neg;
     f->cmp = poly_cmp;
@@ -724,7 +736,7 @@ void trial_divide(darray_ptr factor, darray_ptr mult, mpz_t n)
 	}
 	mpz_nextprime(p, p);
     }
-    
+   
     mpz_clear(m);
     mpz_clear(p);
 }
@@ -782,7 +794,7 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
     element_init(xpow, rxmod);
     element_init(x, rxmod);
     n = factor->count;
-    poly_setx(x); 
+    poly_setx(x);
 
     res = 0;
     for (i=0; i<n; i++) {
